function [vol group] = micca(input,mask,outdir,thresh)
%
% [vol group] = micca(input,mask,outdir,[thresh])
%
% creation du repertoire de sortie
if ~exist(outdir,'file')
    mkdir(outdir)
end

%% lecture des fichiers d'entree
input = dataset('file',input,'readvarnames',false);
mask = dataset('file',mask,'readvarnames',false);

[tmp, tmp, indx_sample] = unique(input.Var1); %#ok<ASGLU>
indx_sample = indx_sample';

indx_ic = input.Var2;
ICs = input.Var3;
mask = mask.Var1;
%% creation du masque commun
mask_all = spm_vol(char(mask)); 
mask = spm_read_vols(mask_all);
new_mask = mask_all(1);
new_mask.fname = sprintf('%s/mask.nii',outdir);
spm_write_vol(new_mask,(sum(mask,4)/size(mask,4))>0.05);
%% lecture des données
[mask XYZmm] = spm_read_vols(spm_vol(new_mask.fname));
[X Y Z] = ind2sub(size(mask),find(mask>0));
vol = spm_vol(char(ICs)); 
data = spm_get_data(vol,[X Y Z]');
%% similarite entre les composantes
data_corr = micca_fastcorr(data');
%% classification
if nargin > 3
    [group_step1 group_step2 group_step3 thresh] = micca_clustering(data, data_corr, indx_sample, thresh); %#ok<NASGU>
else
    [group_step1 group_step2 group_step3 thresh] = micca_clustering(data, data_corr, indx_sample); %#ok<NASGU>
end
%% carte de T
group = group_step3;
Tmap = zeros(max(group),size(data,2));
for g = 1:max(group)
    % check flip IC
    X = find(group == g);
    [~,X1] = max(sum(abs(data_corr(X,X))));
    flip = (data_corr(X,X(X1))>0)*2-1;
    if sum(flip)<0, flip = -flip; end
    pop = bsxfun(@times,data(group == g,:),flip);
    Tmap(g,:) = mean(pop) ./ (std(pop) / sqrt(size(pop,1)));
end
Tmap(isnan(Tmap)) = 0;
%% Write Tmaps
Tmap4D = zeros([vol(1).dim max(group)]);
XYZ = vol(1).mat\[XYZmm(:,mask(:)>0); ones(1,sum(mask(:)>0))];
for i = 1:size(Tmap,2)
    Tmap4D(XYZ(1,i),XYZ(2,i),XYZ(3,i),:) = Tmap(:,i);
end
ni         = nifti;
ni.dat     = file_array([outdir '/micca_groups_tmap.nii'],[vol(1).dim max(group)],'FLOAT32-LE',0,1,0);
ni.mat     = vol(1).mat;
ni.mat0    = vol(1).mat;
ni.descrip = '';
create(ni);
ni.dat(:,:,:,:) = Tmap4D;
%% ecriture de la valeur du seuil
save([outdir '/micca_threshold.txt'],'-ascii','thresh')
%% ecriture des groupes
export(dataset(input.Var1, indx_ic, group_step1', group_step2', group_step3'),'file',[outdir '/micca_groups.txt'],'WriteVarNames',false);
%% return vol
vol = spm_vol([outdir '/micca_groups_tmap.nii']);
