function [group_step1 group_step2 group_step3 thresh] = micca_clustering(data, data_corr, indx_sample, threshold)
%
% [group_step1 group_step2 group_step3 thresh] = micca_clustering(data, data_corr, indx_sample, threshold)
%
% clustering method
%
N = length(unique(indx_sample));

if nargin < 4
    %% automatic threshold estimation
    max_corr = max(abs(data_corr-diag(diag(data_corr))),[],2);
    max_corr_fit = gmdistribution.fit(max_corr,2);
    bins = 0.001;
    X = 0:bins:1;
    if(max_corr_fit.mu(1)>max_corr_fit.mu(2))
        mu1 = max_corr_fit.mu(2);
        Y = max_corr_fit.PComponents(2)*normpdf(X,max_corr_fit.mu(2),sqrt(max_corr_fit.Sigma(:,:,2))) - max_corr_fit.PComponents(1)*normpdf(X,max_corr_fit.mu(1),sqrt(max_corr_fit.Sigma(:,:,1)));
    else
        mu1 = max_corr_fit.mu(1);
        Y = max_corr_fit.PComponents(1)*normpdf(X,max_corr_fit.mu(1),sqrt(max_corr_fit.Sigma(:,:,1))) - max_corr_fit.PComponents(2)*normpdf(X,max_corr_fit.mu(2),sqrt(max_corr_fit.Sigma(:,:,2)));
    end
    thresh = X(find(Y<0 & X>mu1, 1, 'first'));
else
    thresh = threshold;
end

%% compute full connected set
group = micca_full_connected_set(data_corr,indx_sample,thresh);
group = micca_sort_group(group,N*0.05);
group_step1 = group;
%% full connected set iterativ agregation
group = group_step1;
% compute mean map of each set
mean_map = zeros(max(group),size(data,2));
for g = 1:max(group)
    % check flip IC
    X = find(group == g);
    [~,X1] = max(sum(abs(data_corr(X,X))));
    flip = (data_corr(X,X(X1))>0)*2-1;
    if sum(flip)<0, flip = -flip; end
    mean_map(g,:) = mean(bsxfun(@times,data(group == g,:),flip));
end
% compute spatial correlation
mean_map_corr = abs(micca_fastcorr(mean_map'));
% max correlation ?
mean_map_corr = mean_map_corr - eye(size(mean_map_corr));
[m i] = max(mean_map_corr(:));
[g1 g2] = ind2sub(size(mean_map_corr),i);
while m > thresh
    n_commun = sum(ismember(indx_sample(group == g1),indx_sample(group == g2)));
    if n_commun <= ((0.05/(thresh - 1)) * (thresh - m) * N) % m = thresh --> n_commun <= 0 ; m = 1 --> n_commun < 0.05*N ;
        group(group == g2) = g1;
        mean_map(g2,:) = NaN;
        X = find(group == g1);
        [~,X1] = max(sum(abs(data_corr(X,X))));
        flip = (data_corr(X,X(X1))>0)*2-1;
        if sum(flip)<0, flip = -flip; end
        mean_map(g1,:) = mean(bsxfun(@times,data(group == g1,:),flip));
        G = ismember(1:size(mean_map_corr,1),unique(group));
        mean_map_corr(:,g1) = NaN; mean_map_corr(g1,:) = NaN;
        mean_map_corr(:,g2) = NaN; mean_map_corr(g2,:) = NaN;
        mean_map_corr(g1,G) = abs(micca_fastcorr(mean_map(g1,:)',mean_map(G,:)'));
        mean_map_corr(:,g1) = mean_map_corr(g1,:);
        mean_map_corr(g1,g1) = NaN;
    else
        mean_map_corr(g1,g2) = NaN;
        mean_map_corr(g2,g1) = NaN;
    end
    [m i] = max(mean_map_corr(:));
    [g1 g2] = ind2sub(size(mean_map_corr),i);
end
for g = 1:max(group)
    stats = regionprops(indx_sample.*(group == g), 'Area');
    group((group == g) & ismember(indx_sample, find([stats.Area]>1))) = 0;
end
group = micca_sort_group(group,N*0.05);
group_step2 = group;
%% step 3
group = group_step2;
% suppression de la diagonale de la matrice de correlation
data_corr = abs(data_corr - diag(diag(data_corr)));
% retrait des faux positifs
old_group = 0*group;
while any(old_group ~= group)
    old_group = group;
    tmp_corr = data_corr; tmp_corr(group == 0,:) = 0;
    [m,i] = max(tmp_corr); %#ok<ASGLU>
    group(group(i) ~= group & group ~= 0) = 0;
end
group = micca_sort_group(group,N*0.05);
% Ajout des faux négatifs
old_group = 0*group;
while any(old_group ~= group)
    old_group = group;
    tmp_corr = data_corr; tmp_corr(group == 0,:) = 0;
    [rho,match] = max(tmp_corr);
    % on ajoute la composante seulement si m > thresh 
    % si le sujet est deja present on choisit la meilleur des deux composantes
    % on parcours les composantes par maximum de correlation
    [rho IC] = sort(rho.*(group == 0));
    for ic = IC(rho>thresh);
        gn = group(match(ic));
        tmp = find(ismember(indx_sample.*(group == gn),indx_sample(ic)));
        if tmp
            rho2 = max(data_corr(tmp,group == gn));
            if(rho2<rho)
                group(tmp) = 0;
                group(ic) = gn;
            end
        else
            group(ic) = gn;
        end
    end
end
group = micca_sort_group(group,round(N*0.05));
group_step3 = group; 
%hist(nonzeros(group), 1:max(group));
